﻿using SchoolVision.Web.Models.WebLogic;
using Stimulsoft.Report;
using Stimulsoft.Report.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Web.Mvc;
using Microsoft.AspNet.Identity;
using SchoolVision.Core.Api.Commands.Controllers.AdvancedReport;
using SchoolVision.Core.Api.Queries.Controllers.AdvancedReport;
using SchoolVision.Core.Infrastructure.Utilities;
using SchoolVision.Core.Infrastructure.WebSupport.Notification;
using SchoolVision.Web.Models;
using SchoolVision.Web.Models.ControllerViewModels.AdvancedReport;
using Stimulsoft.Report.Web;

namespace SchoolVision.Web.Controllers
{
    public class AdvancedReportController : VisionController
    {
        static AdvancedReportController()
        {
            StiMvcDesigner.CacheHelper = new StiMyCacheHelper();
            //License Key

        }

        public int SelectedReportId {
            get
            {
                if(Session["SelectedReportId"] != null)
                    return Convert.ToInt32(Session["SelectedReportId"]);
                else
                    return -1;
            }
            set => Session["SelectedReportId"] = value;
        }
        public string ReportName {
            get => (string)(Session["ReportName"] ?? "");
            set => Session["ReportName"] = value;
        }
        public bool IsAfterAdd {
            get => (bool)(Session["IsAfterAdd"] ?? false);
            set => Session["IsAfterAdd"] = value;
        }
        // GET: Report  
        public async Task<ActionResult> Index(bool? isPostBack)
        {
            if (isPostBack == null || isPostBack == false)
                SelectedReportId = -1;

            IList<AdvancedReportViewModel> advancedReports = new List<AdvancedReportViewModel>();
            var dbAdvancedReports = await Dispatcher.GetResult(new GetListOfAdvancedReportQuery());
            
            if (dbAdvancedReports.Exception != null && dbAdvancedReports.Exception.IsError)
            {
                var notificationMessage = new NotificationMessage
                {
                    Message = dbAdvancedReports.Exception.ErrorMessage,
                    NotificationType = NotificationType.Danger,
                    PageType = PageType.AdvancedReport_Index,
                    MessageTitle = "Error Occured",
                    UserId = User.Identity.GetUserId<int>()
                };
                NotificationManager.AddNotificationMessage(notificationMessage);
                AddErrorLogEntry(dbAdvancedReports.Exception.ErrorMessage, "AdvancedReport", "Index");
            }
            else
            {
                if(!dbAdvancedReports.AdvancedReports.Any()) return View(advancedReports);
                foreach(var dbAdvancedReport in dbAdvancedReports.AdvancedReports)
                {
                    advancedReports.Add(new AdvancedReportViewModel
                    {
                        AdvancedReportId = dbAdvancedReport.AdvancedReportId,
                        ReportName = dbAdvancedReport.ReportName,
                        ReportDefinition = dbAdvancedReport.ReportDefinition,
                        ModifiedBy = dbAdvancedReport.ModifiedBy,
                        Instant = dbAdvancedReport.Instant,
                        VisionUserName = dbAdvancedReport.VisionUserName
                    });
                }
            }
            ViewBag.SelectedId = SelectedReportId;
            ViewBag.IsAfterAdd = IsAfterAdd;

            return View(advancedReports);
        }
        public ActionResult Edit()
        {
            return View();
        }
        public async Task Delete(string id)
        {
            if(ModelState.IsValid)
            {
                var cmd = new DeleteAdvancedReportCommand { AdvancedReportId = Convert.ToInt32(id) };
                await Dispatcher.Execute(cmd);
                SelectedReportId = -1;
            }
            RedirectToAction("Index");
        }
        [HttpPost]
        public void ChangeViewer(string id)
        {
            IsAfterAdd = false;
            SelectedReportId = Convert.ToInt32(id);
            RedirectToAction("Index");
        }
        public async Task<bool> IsReportNameExist(string reportName)
        {
            var dbReportNames = await Dispatcher.GetResult(new GetAdvancedReportIdAndNameQuery());
            var isExist = dbReportNames.ReportIdAndNames.Any(x => x.ReportName == reportName);
            return isExist;
        }
        public async Task<bool> IsReportNameExist(string reportName, int? reportId)
        {
            var dbReportNames = await Dispatcher.GetResult(new GetAdvancedReportIdAndNameQuery());
            var isExist = dbReportNames.ReportIdAndNames.Any(x => x.ReportName == reportName && x.ReportId != reportId);
            return isExist;
        }
        [HttpPost]
        public async Task<bool> SaveAdvancedReportAsync(string reportName)
        {
            if (string.IsNullOrWhiteSpace(reportName))
                return false;
            var nameIsExist = await IsReportNameExist(reportName.Trim());
            if(nameIsExist)
                return false;
            IsAfterAdd = true;
            var report = new SaveAdvancedReportCommandWithOutput
            {
                AdvancedReportId = SelectedReportId,
                ReportName = reportName.Trim(),
                ModifiedBy = User.Identity.GetUserId<int>(),
                Instant = DateTime.Now,
                IsEditNotCreate = false
            };
            var data = await Dispatcher.ExecuteAndGetOutput(report);
            SelectedReportId = data.AdvancedReportId;
            ReportName = reportName.Trim();
            return SelectedReportId > 0;
        }
    
        //Report Events
        public ActionResult DesignerEvent()
        {
            return StiMvcDesigner.DesignerEventResult();
        }
        public ActionResult ViewerEvent()
        {
            return StiMvcViewer.ViewerEventResult();
        }
        public async Task<ActionResult> GetReporViewertAsync()
        {
            StiReport report = new StiReport();

            if(SelectedReportId == -1 || IsAfterAdd)
                return StiMvcViewer.GetReportResult(report);

            ListOfAdvancedReportQueryResult models = await Dispatcher.GetResult(new GetListOfAdvancedReportQuery());
            var advancedReport = models.AdvancedReports.FirstOrDefault(x => x.AdvancedReportId == SelectedReportId);
            if(advancedReport != null)
            {
                if(advancedReport.ReportDefinition == null)
                    report.ReportName = advancedReport.ReportName;
                else
                    report = getStiReportAsync(advancedReport);

                var advancedReportData = await Dispatcher.GetResult(new GetAdvancedReportDataQuery());
                if(advancedReportData.Exception != null && advancedReportData.Exception.IsError)
                {
                    var notificationMessage = new NotificationMessage
                    {
                        Message = advancedReportData.Exception.ErrorMessage,
                        NotificationType = NotificationType.Danger,
                        PageType = PageType.AdvancedReport_Index,
                        MessageTitle = "Error Occured.",
                        UserId = User.Identity.GetUserId<int>()
                    };
                    NotificationManager.AddNotificationMessage(notificationMessage);
                    AddErrorLogEntry(advancedReportData.Exception.ErrorMessage, "AdvancedReport", "Index");
                }
                else
                {
                    report.Dictionary.Clear();
                    report.RegData(advancedReportData.StudentsData.ToDataSet("Advanced Reporting Domains", "Students"));
                    //report.RegData(advancedReportData.StudentAttendancesData.ToDataSet("Advanced Reporting Domains", "StudentAttendance"));
                    //report.RegData(advancedReportData.StudentBehaviourData.ToDataSet("Advanced Reporting Domains", "StudentBehaviour"));
                    //report.RegData(advancedReportData.StudentAssessmentData.ToDataSet("Advanced Reporting Domains", "StudentAssessment"));
                    report.RegData(advancedReportData.StudentExclusionsData.ToDataSet("Advanced Reporting Domains", "Student Exclusions"));
                    report.RegData(advancedReportData.StaffDetailResultData.ToDataSet("Advanced Reporting Domains", "Staff"));
                    report.RegData(advancedReportData.StaffAbsencesData.ToDataSet("Advanced Reporting Domains", "Staff Absences"));

                    report.Dictionary.Synchronize();
                }
            }

            return StiMvcViewer.GetReportResult(report);
        }
        public async Task<ActionResult> GetReportDesignerAsync()
        {
            StiReport report = new StiReport();
            if(!string.IsNullOrEmpty(ReportName))
                report.ReportName = ReportName;

            var advancedReportData = await Dispatcher.GetResult(new GetAdvancedReportDataQuery());
            if(advancedReportData.Exception != null && advancedReportData.Exception.IsError)
            {
                var notificationMessage = new NotificationMessage
                {
                    Message = advancedReportData.Exception.ErrorMessage,
                    NotificationType = NotificationType.Danger,
                    PageType = PageType.AdvancedReport_Edit,
                    MessageTitle = "Error Occured.",
                    UserId = User.Identity.GetUserId<int>()
                };
                NotificationManager.AddNotificationMessage(notificationMessage);
                AddErrorLogEntry(advancedReportData.Exception.ErrorMessage, "AdvancedReport", "Edit");
            }
            else
            {
                report.Dictionary.Clear();
                report.RegData(advancedReportData.StudentsData.ToDataSet("Advanced Reporting Domains", "Students"));
                //report.RegData(advancedReportData.StudentAttendancesData.ToDataSet("Advanced Reporting Domains", "StudentAttendance"));
                //report.RegData(advancedReportData.StudentBehaviourData.ToDataSet("Advanced Reporting Domains", "StudentBehaviour"));
                //report.RegData(advancedReportData.StudentAssessmentData.ToDataSet("Advanced Reporting Domains", "StudentAssessment"));
                report.RegData(advancedReportData.StudentExclusionsData.ToDataSet("Advanced Reporting Domains", "Student Exclusions"));
                report.RegData(advancedReportData.StaffDetailResultData.ToDataSet("Advanced Reporting Domains", "Staff"));
                report.RegData(advancedReportData.StaffAbsencesData.ToDataSet("Advanced Reporting Domains", "Staff Absences"));

                report.Dictionary.Synchronize();
            }

            if(SelectedReportId == -1)//When click new
                return StiMvcDesigner.GetReportResult(report);

            ListOfAdvancedReportQueryResult models = await Dispatcher.GetResult(new GetListOfAdvancedReportQuery());
            var advancedReport = models.AdvancedReports.FirstOrDefault(x => x.AdvancedReportId == SelectedReportId);
            if(advancedReport != null && advancedReport.ReportDefinition != null)
                report = getStiReportAsync(advancedReport);
            
            return StiMvcDesigner.GetReportResult(report);
        }
        public async Task<ActionResult> SaveReportAsync()
        {
            StiReport stiReport = StiMvcDesigner.GetReportObject();
            var message = "";
            var isEditNotCreate = true;
            if(stiReport != null)
            {
                if (SelectedReportId == -1) isEditNotCreate = false;
                var nameIsExist = SelectedReportId == -1 ? await IsReportNameExist(stiReport.ReportName) :  await IsReportNameExist(stiReport.ReportName, SelectedReportId);
                if(nameIsExist)
                    message = "The report name is already exist";
                else
                {
                    var report = new SaveAdvancedReportCommandWithOutput
                    {
                        AdvancedReportId = SelectedReportId,
                        ReportName = stiReport.ReportName,
                        ReportDefinition = stiReport.SavePackedReportToString(),
                        ModifiedBy = User.Identity.GetUserId<int>(),
                        Instant = DateTime.Now,
                        IsEditNotCreate = isEditNotCreate
                    };
                    var data = await Dispatcher.ExecuteAndGetOutput(report);
                    SelectedReportId = data.AdvancedReportId;
                }
            }
            else
                message = "The report is not found.";

            if(string.IsNullOrEmpty(message))
                return StiMvcDesigner.SaveReportResult();
            return StiMvcDesigner.SaveReportResult(message);
        }
        private StiReport getStiReportAsync(AdvancedReportEntry advancedReport)
        {
            StiReport tempReport = new StiReport();

            if(advancedReport.ReportDefinition != string.Empty)
                tempReport.LoadPackedReportFromString(advancedReport.ReportDefinition);

            tempReport.ReportName = advancedReport.ReportName;
            tempReport.ReportAuthor = advancedReport.ModifiedBy.ToString();
            tempReport.ReportCreated = advancedReport.Instant;

            tempReport.Dictionary.Synchronize();

            return tempReport;
        }
        public async Task<ActionResult> PreviewReportAsync()
        {
            StiReport report = StiMvcDesigner.GetActionReportObject();
            var advancedReportData = await Dispatcher.GetResult(new GetAdvancedReportDataQuery());
            if(advancedReportData.Exception != null && advancedReportData.Exception.IsError)
            {
                var notificationMessage = new NotificationMessage
                {
                    Message = advancedReportData.Exception.ErrorMessage,
                    NotificationType = NotificationType.Danger,
                    PageType = PageType.AdvancedReport_Edit,
                    MessageTitle = "Error Occured.",
                    UserId = User.Identity.GetUserId<int>()
                };
                NotificationManager.AddNotificationMessage(notificationMessage);
                AddErrorLogEntry(advancedReportData.Exception.ErrorMessage, "AdvancedReport", "Edit");
            }
            else
            {
                report.RegData(advancedReportData.StudentsData.ToDataSet("Advanced Reporting Domains", "Students"));
                //report.RegData(advancedReportData.StudentAttendancesData.ToDataSet("Advanced Reporting Domains", "StudentAttendance"));
                //report.RegData(advancedReportData.StudentBehaviourData.ToDataSet("Advanced Reporting Domains", "StudentBehaviour"));
                //report.RegData(advancedReportData.StudentAssessmentData.ToDataSet("Advanced Reporting Domains", "StudentAssessment"));
                report.RegData(advancedReportData.StudentExclusionsData.ToDataSet("Advanced Reporting Domains", "Student Exclusions"));
                report.RegData(advancedReportData.StaffDetailResultData.ToDataSet("Advanced Reporting Domains", "Staff"));
                report.RegData(advancedReportData.StaffAbsencesData.ToDataSet("Advanced Reporting Domains", "Staff Absences"));
            }

            return StiMvcDesigner.PreviewReportResult(report);
        }

        #region CacheHelper
        public class StiMyCacheHelper : StiCacheHelper
        {
            public override StiReport GetReport(string guid)
            {
                var advancedReportCacheFiles = Dispatcher.GetResult(new GetAdvancedReportCacheFilesQuery() { CacheGuid = guid }).Result;
                byte[] packedReport = advancedReportCacheFiles.CacheData;
                if(packedReport != null)
                {
                    StiReport report = new StiReport();

                    if(guid.EndsWith("template"))
                        report.LoadPackedReport(packedReport);
                    else
                        report.LoadPackedDocument(packedReport);

                    return report;
                }
                else if(guid.EndsWith("template"))
                {
                    return new StiReport();
                }
                return null;
            }
            public override void SaveReport(StiReport report, string guid)
            {
                byte[] packedReport = guid.EndsWith("template") ? report.SavePackedReportToByteArray() : report.SavePackedDocumentToByteArray();
                Dispatcher.Execute(new SaveAdvancedReportCacheFilesCommand { CacheGuid = guid, CacheData = packedReport, Instant = DateTime.Now });
            }
            
            public override object GetObject(string guid)
            {
                var advancedReportCacheFiles = Dispatcher.GetResult(new GetAdvancedReportCacheFilesQuery() { CacheGuid = guid }).Result;
                byte[] cacheData = advancedReportCacheFiles.CacheData;
                if(cacheData != null)
                    return StiCacheHelper.GetObjectFromCacheData(cacheData);
                return null;
            }

            public override void SaveObject(object obj, string guid)
            {
                byte[] cacheData = StiCacheHelper.GetCacheDataFromObject(obj);
                Dispatcher.Execute(new SaveAdvancedReportCacheFilesCommand() { CacheGuid = guid, CacheData = cacheData, Instant = DateTime.Now });
            }
        }
        #endregion
    }

}